==> adfurikun-1.8.0.tar.gz: android/src/main/kotlin/jp/co/glossom/adfurikun/AdfurikunPlugin.kt <==
package jp.co.glossom.adfurikun

import android.app.Activity
import androidx.annotation.NonNull

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import jp.tjkapp.adfurikunsdk.moviereward.AdfurikunSdk

/** AdfurikunPlugin */
class AdfurikunPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {

  private lateinit var channel : MethodChannel

  private var nativeAdFactory: AdfurikunNativeAdFactory? = null
  private var rectangleFactory: AdfurikunRectangleFactory? = null
  private var bannerFactory: AdfurikunBannerFactory? = null

  private val rewardMap = hashMapOf<String, AdfurikunReward>()
  private val interstitialMap = hashMapOf<String, AdfurikunInterstitial>()
  private val appOpenAdMap = hashMapOf<String, AdfurikunAppOpenAd>()

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    AdfurikunSdk.init(flutterPluginBinding.applicationContext)
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, Constants.MAIN_METHOD_CHANNEL)
    channel.setMethodCallHandler(this)
    nativeAdFactory = AdfurikunNativeAdFactory(flutterPluginBinding.binaryMessenger, channel)
    nativeAdFactory?.let {
      flutterPluginBinding.platformViewRegistry.registerViewFactory(Constants.NATIVE_AD_VIEW_TYPE_ID, it)
    }
    rectangleFactory = AdfurikunRectangleFactory(flutterPluginBinding.binaryMessenger, channel)
    rectangleFactory?.let {
      flutterPluginBinding.platformViewRegistry.registerViewFactory(Constants.RECTANGLE_VIEW_TYPE_ID, it)
    }
    bannerFactory = AdfurikunBannerFactory(flutterPluginBinding.binaryMessenger, channel)
    bannerFactory?.let {
      flutterPluginBinding.platformViewRegistry.registerViewFactory(Constants.BANNER_VIEW_TYPE_ID, it)
    }
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    val arguments = (call.arguments as? HashMap<String, *>)
    when (call.method) {
      // アプリ起動時
      Constants.INIT_APP_OPEN_AD -> initAppOpenAd(arguments)
      Constants.LOAD_APP_OPEN_AD -> loadAppOpenAd(arguments)
      Constants.IS_PREPARED_APP_OPEN_AD -> isPreparedAppOpenAd(arguments, result)
      Constants.PLAY_APP_OPEN_AD -> playAppOpenAd(arguments)
      Constants.DESTROY_APP_OPEN_AD -> destroyAppOpenAd(arguments)
      // リワード
      Constants.INIT_REWARD -> initReward(arguments)
      Constants.LOAD_REWARD -> loadReward(arguments)
      Constants.IS_PREPARED_REWARD -> isPreparedReward(arguments, result)
      Constants.PLAY_REWARD -> playReward(arguments)
      Constants.DESTROY_REWARD -> destroyReward(arguments)
      // インタースティシャル
      Constants.INIT_INTERSTITIAL -> initInterstitial(arguments)
      Constants.LOAD_INTERSTITIAL -> loadInterstitial(arguments)
      Constants.IS_PREPARED_INTERSTITIAL -> isPreparedInterstitial(arguments, result)
      Constants.PLAY_INTERSTITIAL -> playInterstitial(arguments)
      Constants.DESTROY_INTERSTITIAL -> destroyInterstitial(arguments)
      // ネイティブ
      Constants.INIT_NATIVE_AD,
      Constants.LOAD_NATIVE_AD,
      Constants.PLAY_NATIVE_AD,
      Constants.DESTROY_NATIVE_AD -> nativeAdFactory?.onMethodCall(call, result)
      // レクタングル
      Constants.INIT_RECTANGLE,
      Constants.LOAD_RECTANGLE,
      Constants.PLAY_RECTANGLE,
      Constants.DESTROY_RECTANGLE -> rectangleFactory?.onMethodCall(call, result)
      // バナー
      Constants.INIT_BANNER,
      Constants.LOAD_BANNER,
      Constants.PLAY_BANNER,
      Constants.DESTROY_BANNER -> bannerFactory?.onMethodCall(call, result)
      // 音
      Constants.SET_SOUND_STATE -> AdfurikunCommons.setSoundState(arguments)
      // テストモード
      Constants.SET_TEST_MODE -> AdfurikunCommons.setTestMode(arguments)
      // ユーザーGDPR同意
      Constants.SET_USER_CONSENT -> AdfurikunCommons.setUserConsent(arguments)
      // COPPA
      Constants.SET_CHILD_DIRECTED -> AdfurikunCommons.setChildDirected(arguments)
      // Googleファミリーポリシー
      Constants.SET_GOOGLE_FAMILIES_POLICY -> AdfurikunCommons.setGoogleFamiliesPolicy(arguments)
      // 地域
      Constants.SET_REGION -> AdfurikunCommons.setRegion(arguments)
      // 分析用データ送信
      Constants.FORCE_SEND_EVENT -> AdfurikunCommons.forceSendEvent()
      // 配信情報修正
      Constants.SET_AD_NETWORK_INFORMATION -> AdfurikunCommons.setAdNetworkInformation(arguments)
      else -> result.notImplemented()
    }
  }

  /**
   * 初期化（アプリ起動時）
   * @param arguments
   */
  private fun initAppOpenAd(arguments: HashMap<String, *>?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && number >= 0 && !appOpenAdMap.containsKey(appId)) {
      AdfurikunCommons.setPluginVersion(arguments)
      val appOpenAd = AdfurikunAppOpenAd(number, appId, channel)
      val appLogoIconName = (arguments?.get(Constants.KEY_APP_LOGO_ICON_NAME) as? String) ?: ""
      appOpenAd.init(appLogoIconName)
      appOpenAdMap[appId] = appOpenAd
    }
  }

  /**
   * 読み込み（アプリ起動時）
   * @param arguments
   */
  private fun loadAppOpenAd(arguments: HashMap<String, *>?) {
    checkAppOpenAdArgument(arguments) { appId, number ->
      appOpenAdMap[appId]?.let {
        it.number = number
        it.load(AdfurikunCommons.getLoadWithTimeoutDouble(arguments))
      }
    }
  }

  /**
   * 準備判定（アプリ起動時）
   * @param arguments
   * @param result
   */
  private fun isPreparedAppOpenAd(arguments: HashMap<String, *>?, result: Result) {
    checkAppOpenAdArgument(arguments) { appId, _ ->
      appOpenAdMap[appId]?.isPrepared(result)
    }
  }

  /**
   * 再生（アプリ起動時）
   * @param arguments
   */
  private fun playAppOpenAd(arguments: HashMap<String, *>?) {
    checkAppOpenAdArgument(arguments) { appId, number ->
      appOpenAdMap[appId]?.let {
        it.number = number
        it.play()
      }
    }
  }

  /**
   * 破棄（アプリ起動時）
   * @param arguments
   */
  private fun destroyAppOpenAd(arguments: HashMap<String, *>?) {
    checkAppOpenAdArgument(arguments) { appId, _ ->
      appOpenAdMap.remove(appId)?.destroy()
    }
  }

  /**
   * Argument判定（アプリ起動時）
   * @param arguments
   * @param result
   */
  private fun checkAppOpenAdArgument(arguments: HashMap<String, *>?, result: ((String?, Int) -> Unit)?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && appOpenAdMap.containsKey(appId) && number >= 0) {
      result?.invoke(appId, number)
    }
  }

  /**
   * 初期化（リワード）
   * @param arguments
   */
  private fun initReward(arguments: HashMap<String, *>?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && number >= 0 && !rewardMap.containsKey(appId)) {
      AdfurikunCommons.setPluginVersion(arguments)
      val reward = AdfurikunReward(number, appId, channel)
      reward.init()
      rewardMap[appId] = reward
    }
  }

  /**
   * 読み込み（リワード）
   * @param arguments
   */
  private fun loadReward(arguments: HashMap<String, *>?) {
    checkRewardArgument(arguments) { appId, number ->
      rewardMap[appId]?.let {
        it.number = number
        it.load(AdfurikunCommons.getLoadWithTimeout(arguments))
      }
    }
  }

  /**
   * 準備判定（リワード）
   * @param arguments
   * @param result
   */
  private fun isPreparedReward(arguments: HashMap<String, *>?, result: Result) {
    checkRewardArgument(arguments) { appId, _ ->
      rewardMap[appId]?.isPrepared(result)
    }
  }

  /**
   * 再生（リワード）
   * @param arguments
   */
  private fun playReward(arguments: HashMap<String, *>?) {
    checkRewardArgument(arguments) { appId, number ->
      rewardMap[appId]?.let {
        it.number = number
        it.play(convertCustomParams(arguments))
      }
    }
  }

  /**
   * 破棄（リワード）
   * @param arguments
   */
  private fun destroyReward(arguments: HashMap<String, *>?) {
    checkRewardArgument(arguments) { appId, _ ->
      rewardMap.remove(appId)?.destroy()
    }
  }

  /**
   * Argument判定（リワード）
   * @param arguments
   * @param result
   */
  private fun checkRewardArgument(arguments: HashMap<String, *>?, result: ((String?, Int) -> Unit)?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && rewardMap.containsKey(appId) && number >= 0) {
      result?.invoke(appId, number)
    }
  }

  /**
   * 初期化（インタースティシャル）
   * @param arguments
   */
  private fun initInterstitial(arguments: HashMap<String, *>?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && number >= 0 && !interstitialMap.containsKey(appId)) {
      AdfurikunCommons.setPluginVersion(arguments)
      val interstitial = AdfurikunInterstitial(number, appId, channel)
      interstitial.init()
      interstitialMap[appId] = interstitial
    }
  }

  /**
   * 読み込み（インタースティシャル）
   * @param arguments
   */
  private fun loadInterstitial(arguments: HashMap<String, *>?) {
    checkInterstitialArgument(arguments) { appId, number ->
      interstitialMap[appId]?.let {
        it.number = number
        it.load(AdfurikunCommons.getLoadWithTimeout(arguments))
      }
    }
  }

  /**
   * 準備判定（インタースティシャル）
   * @param arguments
   * @param result
   */
  private fun isPreparedInterstitial(arguments: HashMap<String, *>?, result: Result) {
    checkInterstitialArgument(arguments) { appId, _ ->
      interstitialMap[appId]?.isPrepared(result)
    }
  }

  /**
   * 再生（インタースティシャル）
   * @param arguments
   */
  private fun playInterstitial(arguments: HashMap<String, *>?) {
    checkInterstitialArgument(arguments) { appId, number ->
      interstitialMap[appId]?.let {
        it.number = number
        it.play(convertCustomParams(arguments))
      }
    }
  }

  /**
   * 破棄（インタースティシャル）
   * @param arguments
   */
  private fun destroyInterstitial(arguments: HashMap<String, *>?) {
    checkInterstitialArgument(arguments) { appId, _ ->
      interstitialMap.remove(appId)?.destroy()
    }
  }

  /**
   * Argument判定（インタースティシャル）
   * @param arguments
   * @param result
   */
  private fun checkInterstitialArgument(arguments: HashMap<String, *>?, result: ((String?, Int) -> Unit)?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && interstitialMap.containsKey(appId) && number >= 0) {
      result?.invoke(appId, number)
    }
  }

  /**
   * CustomParamsを変換
   * @param arguments
   */
  private fun convertCustomParams(arguments: HashMap<String, *>?): HashMap<String, String>? {
    (arguments?.get(Constants.KEY_CUSTOM_PARAMS) as? HashMap<String, String>)?.let {
      if (it.isNotEmpty()) {
        return it
      }
    }
    return null
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    currentActivity = binding.activity
  }

  override fun onDetachedFromActivity() {
    // 処理なし
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    // 処理なし
  }

  override fun onDetachedFromActivityForConfigChanges() {
    // 処理なし
  }

  companion object {
    internal var currentActivity: Activity? = null
  }
}

==> adfurikun-1.9.0.tar.gz: android/src/main/kotlin/jp/co/glossom/adfurikun/AdfurikunPlugin.kt <==
package jp.co.glossom.adfurikun

import android.app.Activity
import androidx.annotation.NonNull

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import jp.tjkapp.adfurikunsdk.moviereward.AdfurikunSdk

/** AdfurikunPlugin */
class AdfurikunPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {

  private lateinit var channel : MethodChannel

  private var nativeAdFactory: AdfurikunNativeAdFactory? = null
  private var rectangleFactory: AdfurikunRectangleFactory? = null
  private var bannerFactory: AdfurikunBannerFactory? = null

  private val rewardMap = hashMapOf<String, AdfurikunReward>()
  private val interstitialMap = hashMapOf<String, AdfurikunInterstitial>()
  private val appOpenAdMap = hashMapOf<String, AdfurikunAppOpenAd>()

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    AdfurikunSdk.init(flutterPluginBinding.applicationContext)
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, Constants.MAIN_METHOD_CHANNEL)
    channel.setMethodCallHandler(this)
    nativeAdFactory = AdfurikunNativeAdFactory(flutterPluginBinding.binaryMessenger, channel)
    nativeAdFactory?.let {
      flutterPluginBinding.platformViewRegistry.registerViewFactory(Constants.NATIVE_AD_VIEW_TYPE_ID, it)
    }
    rectangleFactory = AdfurikunRectangleFactory(flutterPluginBinding.binaryMessenger, channel)
    rectangleFactory?.let {
      flutterPluginBinding.platformViewRegistry.registerViewFactory(Constants.RECTANGLE_VIEW_TYPE_ID, it)
    }
    bannerFactory = AdfurikunBannerFactory(flutterPluginBinding.binaryMessenger, channel)
    bannerFactory?.let {
      flutterPluginBinding.platformViewRegistry.registerViewFactory(Constants.BANNER_VIEW_TYPE_ID, it)
    }
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    val arguments = (call.arguments as? HashMap<String, *>)
    when (call.method) {
      // アプリ起動時
      Constants.INIT_APP_OPEN_AD -> initAppOpenAd(arguments)
      Constants.LOAD_APP_OPEN_AD -> loadAppOpenAd(arguments)
      Constants.IS_PREPARED_APP_OPEN_AD -> isPreparedAppOpenAd(arguments, result)
      Constants.PLAY_APP_OPEN_AD -> playAppOpenAd(arguments)
      Constants.DESTROY_APP_OPEN_AD -> destroyAppOpenAd(arguments)
      // リワード
      Constants.INIT_REWARD -> initReward(arguments)
      Constants.SET_TRACKING_ID_REWARD -> setTrackingIdReward(arguments)
      Constants.LOAD_REWARD -> loadReward(arguments)
      Constants.IS_PREPARED_REWARD -> isPreparedReward(arguments, result)
      Constants.PLAY_REWARD -> playReward(arguments)
      Constants.DESTROY_REWARD -> destroyReward(arguments)
      Constants.REWARD_COMPLETED_REWARD -> rewardCompletedReward(arguments)
      Constants.NOT_READY_ALERT_REWARD -> notReadyAlertReward(arguments)
      Constants.SHOW_AD_PLAY_BUTTON_REWARD -> showAdPlayButtonReward(arguments)
      Constants.CLICK_AD_PLAY_BUTTON_REWARD -> clickAdPlayButtonReward(arguments)
      // インタースティシャル
      Constants.INIT_INTERSTITIAL -> initInterstitial(arguments)
      Constants.SET_TRACKING_ID_INTERSTITIAL -> setTrackingIdInterstitial(arguments)
      Constants.LOAD_INTERSTITIAL -> loadInterstitial(arguments)
      Constants.IS_PREPARED_INTERSTITIAL -> isPreparedInterstitial(arguments, result)
      Constants.PLAY_INTERSTITIAL -> playInterstitial(arguments)
      Constants.DESTROY_INTERSTITIAL -> destroyInterstitial(arguments)
      // ネイティブ
      Constants.INIT_NATIVE_AD,
      Constants.SET_TRACKING_ID_NATIVE_AD,
      Constants.LOAD_NATIVE_AD,
      Constants.PLAY_NATIVE_AD,
      Constants.DESTROY_NATIVE_AD -> nativeAdFactory?.onMethodCall(call, result)
      // レクタングル
      Constants.INIT_RECTANGLE,
      Constants.SET_TRACKING_ID_RECTANGLE,
      Constants.LOAD_RECTANGLE,
      Constants.PLAY_RECTANGLE,
      Constants.DESTROY_RECTANGLE -> rectangleFactory?.onMethodCall(call, result)
      // バナー
      Constants.INIT_BANNER,
      Constants.SET_TRACKING_ID_BANNER,
      Constants.LOAD_BANNER,
      Constants.PLAY_BANNER,
      Constants.DESTROY_BANNER -> bannerFactory?.onMethodCall(call, result)
      // 音
      Constants.SET_SOUND_STATE -> AdfurikunCommons.setSoundState(arguments)
      // テストモード
      Constants.SET_TEST_MODE -> AdfurikunCommons.setTestMode(arguments)
      // デバッグモード
      Constants.SET_DEBUG_MODE -> AdfurikunCommons.setDebugMode(arguments)
      // ユーザーGDPR同意
      Constants.SET_USER_CONSENT -> AdfurikunCommons.setUserConsent(arguments)
      // COPPA
      Constants.SET_CHILD_DIRECTED -> AdfurikunCommons.setChildDirected(arguments)
      // Googleファミリーポリシー
      Constants.SET_GOOGLE_FAMILIES_POLICY -> AdfurikunCommons.setGoogleFamiliesPolicy(arguments)
      // 地域
      Constants.SET_REGION -> AdfurikunCommons.setRegion(arguments)
      // 分析用データ送信
      Constants.FORCE_SEND_EVENT -> AdfurikunCommons.forceSendEvent()
      // 配信情報修正
      Constants.SET_AD_NETWORK_INFORMATION -> AdfurikunCommons.setAdNetworkInformation(arguments)
      // アプリログを送信
      Constants.SEND_APPLICATION_LOG -> AdfurikunCommons.sendApplicationLog(arguments)
      // アドフリくんSDKバージョン
      Constants.VERSION -> AdfurikunCommons.version(result)
      else -> result.notImplemented()
    }
  }

  /**
   * 初期化（アプリ起動時）
   * @param arguments
   */
  private fun initAppOpenAd(arguments: HashMap<String, *>?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && number >= 0 && !appOpenAdMap.containsKey(appId)) {
      AdfurikunCommons.setPluginVersion(arguments)
      val appOpenAd = AdfurikunAppOpenAd(number, appId, channel)
      val appLogoIconName = (arguments?.get(Constants.KEY_APP_LOGO_ICON_NAME) as? String) ?: ""
      appOpenAd.init(appLogoIconName)
      appOpenAdMap[appId] = appOpenAd
    }
  }

  /**
   * 読み込み（アプリ起動時）
   * @param arguments
   */
  private fun loadAppOpenAd(arguments: HashMap<String, *>?) {
    checkAppOpenAdArgument(arguments) { appId, number ->
      appOpenAdMap[appId]?.let {
        it.number = number
        it.load(AdfurikunCommons.getLoadWithTimeoutDouble(arguments))
      }
    }
  }

  /**
   * 準備判定（アプリ起動時）
   * @param arguments
   * @param result
   */
  private fun isPreparedAppOpenAd(arguments: HashMap<String, *>?, result: Result) {
    checkAppOpenAdArgument(arguments) { appId, _ ->
      appOpenAdMap[appId]?.isPrepared(result)
    }
  }

  /**
   * 再生（アプリ起動時）
   * @param arguments
   */
  private fun playAppOpenAd(arguments: HashMap<String, *>?) {
    checkAppOpenAdArgument(arguments) { appId, number ->
      appOpenAdMap[appId]?.let {
        it.number = number
        it.play()
      }
    }
  }

  /**
   * 破棄（アプリ起動時）
   * @param arguments
   */
  private fun destroyAppOpenAd(arguments: HashMap<String, *>?) {
    checkAppOpenAdArgument(arguments) { appId, _ ->
      appOpenAdMap.remove(appId)?.destroy()
    }
  }

  /**
   * Argument判定（アプリ起動時）
   * @param arguments
   * @param result
   */
  private fun checkAppOpenAdArgument(arguments: HashMap<String, *>?, result: ((String?, Int) -> Unit)?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && appOpenAdMap.containsKey(appId) && number >= 0) {
      result?.invoke(appId, number)
    }
  }

  /**
   * 初期化（リワード）
   * @param arguments
   */
  private fun initReward(arguments: HashMap<String, *>?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && number >= 0 && !rewardMap.containsKey(appId)) {
      AdfurikunCommons.setPluginVersion(arguments)
      val reward = AdfurikunReward(number, appId, channel)
      reward.init()
      rewardMap[appId] = reward
    }
  }

  /**
   * 読み込み（リワード）
   * @param arguments
   */
  private fun loadReward(arguments: HashMap<String, *>?) {
    checkRewardArgument(arguments) { appId, number ->
      rewardMap[appId]?.let {
        it.number = number
        it.load(AdfurikunCommons.getLoadWithTimeout(arguments))
      }
    }
  }

  /**
   * トラッキングIDを設定（リワード）
   * @param arguments
   */
  private fun setTrackingIdReward(arguments: HashMap<String, *>?) {
    checkRewardArgument(arguments) { appId, number ->
      rewardMap[appId]?.let {
        it.number = number
        it.setTrackingId(AdfurikunCommons.convertTrackingId(arguments))
      }
    }
  }

  /**
   * 準備判定（リワード）
   * @param arguments
   * @param result
   */
  private fun isPreparedReward(arguments: HashMap<String, *>?, result: Result) {
    checkRewardArgument(arguments) { appId, _ ->
      rewardMap[appId]?.isPrepared(result)
    }
  }

  /**
   * 再生（リワード）
   * @param arguments
   */
  private fun playReward(arguments: HashMap<String, *>?) {
    checkRewardArgument(arguments) { appId, number ->
      rewardMap[appId]?.let {
        it.number = number
        it.play(AdfurikunCommons.convertCustomParams(arguments))
      }
    }
  }

  /**
   * 破棄（リワード）
   * @param arguments
   */
  private fun destroyReward(arguments: HashMap<String, *>?) {
    checkRewardArgument(arguments) { appId, _ ->
      rewardMap.remove(appId)?.destroy()
    }
  }

  /**
   * リワード成功有無をアプリから通知（リワード）
   * @param arguments
   */
  private fun rewardCompletedReward(arguments: HashMap<String, *>?) {
    checkRewardArgument(arguments) { appId, number ->
      rewardMap[appId]?.let {
        it.number = number
        it.rewardCompleted((arguments?.get(Constants.KEY_BOOL) as? Boolean) ?: false)
      }
    }
  }

  /**
   * 在庫なし状態をアプリから通知（リワード）
   * @param arguments
   */
  private fun notReadyAlertReward(arguments: HashMap<String, *>?) {
    checkRewardArgument(arguments) { appId, number ->
      rewardMap[appId]?.let {
        it.number = number
        it.notReadyAlert()
      }
    }
  }

  /**
   * 広告を再生するボタンが画面に表示された事をアプリから通知（リワード）
   * @param arguments
   */
  private fun showAdPlayButtonReward(arguments: HashMap<String, *>?) {
    checkRewardArgument(arguments) { appId, number ->
      rewardMap[appId]?.let {
        it.number = number
        it.showAdPlayButton()
      }
    }
  }

  /**
   * ユーザーが"CMを観る"ボタンを押したとき事をアプリから通知（リワード）
   * @param arguments
   */
  private fun clickAdPlayButtonReward(arguments: HashMap<String, *>?) {
    checkRewardArgument(arguments) { appId, number ->
      rewardMap[appId]?.let {
        it.number = number
        it.clickAdPlayButton()
      }
    }
  }

  /**
   * Argument判定（リワード）
   * @param arguments
   * @param result
   */
  private fun checkRewardArgument(arguments: HashMap<String, *>?, result: ((String?, Int) -> Unit)?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && rewardMap.containsKey(appId) && number >= 0) {
      result?.invoke(appId, number)
    }
  }

  /**
   * 初期化（インタースティシャル）
   * @param arguments
   */
  private fun initInterstitial(arguments: HashMap<String, *>?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && number >= 0 && !interstitialMap.containsKey(appId)) {
      AdfurikunCommons.setPluginVersion(arguments)
      val interstitial = AdfurikunInterstitial(number, appId, channel)
      interstitial.init()
      interstitialMap[appId] = interstitial
    }
  }

  /**
   * トラッキングIDを設定（インタースティシャル）
   * @param arguments
   */
  private fun setTrackingIdInterstitial(arguments: HashMap<String, *>?) {
    checkInterstitialArgument(arguments) { appId, number ->
      interstitialMap[appId]?.let {
        it.number = number
        it.setTrackingId(AdfurikunCommons.convertTrackingId(arguments))
      }
    }
  }

  /**
   * 読み込み（インタースティシャル）
   * @param arguments
   */
  private fun loadInterstitial(arguments: HashMap<String, *>?) {
    checkInterstitialArgument(arguments) { appId, number ->
      interstitialMap[appId]?.let {
        it.number = number
        it.load(AdfurikunCommons.getLoadWithTimeout(arguments))
      }
    }
  }

  /**
   * 準備判定（インタースティシャル）
   * @param arguments
   * @param result
   */
  private fun isPreparedInterstitial(arguments: HashMap<String, *>?, result: Result) {
    checkInterstitialArgument(arguments) { appId, _ ->
      interstitialMap[appId]?.isPrepared(result)
    }
  }

  /**
   * 再生（インタースティシャル）
   * @param arguments
   */
  private fun playInterstitial(arguments: HashMap<String, *>?) {
    checkInterstitialArgument(arguments) { appId, number ->
      interstitialMap[appId]?.let {
        it.number = number
        it.play(AdfurikunCommons.convertCustomParams(arguments))
      }
    }
  }

  /**
   * 破棄（インタースティシャル）
   * @param arguments
   */
  private fun destroyInterstitial(arguments: HashMap<String, *>?) {
    checkInterstitialArgument(arguments) { appId, _ ->
      interstitialMap.remove(appId)?.destroy()
    }
  }

  /**
   * Argument判定（インタースティシャル）
   * @param arguments
   * @param result
   */
  private fun checkInterstitialArgument(arguments: HashMap<String, *>?, result: ((String?, Int) -> Unit)?) {
    val number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
    val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
    if (appId.isNotBlank() && interstitialMap.containsKey(appId) && number >= 0) {
      result?.invoke(appId, number)
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    currentActivity = binding.activity
  }

  override fun onDetachedFromActivity() {
    // 処理なし
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    // 処理なし
  }

  override fun onDetachedFromActivityForConfigChanges() {
    // 処理なし
  }

  companion object {
    internal var currentActivity: Activity? = null
  }
}
